import { displayNumber } from '../../util/helpers.js';

export class Vertex {
  constructor(graph, type, id, data, options = {}) {
    this.graph = graph;
    this.type = type;
    this.id = id;
    this.data = data;
    this.label = options.label ?? this.id;
    this.options = options;
    this.edges = {
      from: [],
      to: [],
    };
    this.installedClickCallback = false;
    this.properties = new Map();
  }

  reset() {
    this.installedClickCallback = false;
  }

  getEdges(type, away) {
    return this.edges[away ? 'from' : 'to'].filter(
      (edge) => edge.type === type,
    );
  }

  setProperty(key, value) {
    this.properties.set(key, value);
    return this;
  }

  displayVertex() {
    if (this.options.hide) {
      return;
    }

    let html = `${this.label}`;
    html += '<table>';
    for (const [key, value] of this.properties.entries()) {
      const displayValue = typeof value === 'number' ? displayNumber(value) : value;
      html += `<tr><td>${key}</td><td>${displayValue}</td></tr>`;
    }
    html += '</table>';
    html = html.replaceAll(/\n\s*/g, '');
    this.graph.flowchart?.log(`${this.id}[${html}]`);

    if (this.graph.editable && !this.installedClickCallback) {
      this.graph.flowchart?.log(`click ${this.id} WDGHandler${this.graph.index} "Edit Vertex ${this.id}"`);
      this.installedClickCallback = true;
    }
  }

  static prepareEditorDocument(graph, doc, vertexId) {
    doc.clear();
    const vertex = graph.getVertex(vertexId);
    if (!vertex) {
      throw new Error(`Could not find WDG Vertex ${vertexId}`);
    }
    const form = doc.form().lastElement;
    doc.remark('<h3>Edit Vertex</h3>', { parentEl: form.el });
    form
      .textField({
        id: 'id', name: 'id', defaultValue: vertex.id,
      })
      .textField({ id: 'type', name: 'type', defaultValue: vertex.type })
      .textField({ id: 'label', name: 'label', defaultValue: vertex.label });

    doc.remark('<h4>Properties</h4>', { parentEl: form.el });
    const subFormArray = form.subFormArray({ id: 'properties', name: 'properties' }).lastItem;
    const addPropertyForm = (key, value) => {
      const { subForm } = form.subForm({ name: 'subform', subFormArray }).lastItem;
      subForm.textField({ id: 'key', name: 'key', defaultValue: key })
        .textField({ id: 'value', name: 'value', defaultValue: value })
        .button({
          id: 'remove',
          name: 'Remove Property',
          cb: () => subFormArray.remove(subForm),
        });
      doc.remark('<br>', { parentEl: subForm.el });
    };

    for (const [key, value] of vertex.properties.entries()) {
      addPropertyForm(key, value);
    }

    form.button({
      id: 'add',
      name: 'Add Property',
      cb: () => addPropertyForm('', ''),
    })
      .button({
        id: 'save',
        name: 'Save',
        type: 'submit',
        cb: ({ form: { value: formValue } }) => {
          let fullRedraw = false;
          if (formValue.id !== vertex.id) {
            fullRedraw = true;
          }
          // TODO: preserve data types of properties
          formValue.properties = new Map(formValue.properties.map(({ key, value }) => [key, value]));
          Object.assign(vertex, formValue);
          vertex.displayVertex();
          if (fullRedraw) {
            graph.redraw();
          }
        },
      })
      .button({
        id: 'cancel',
        name: 'Cancel',
        cb: () => doc.clear(),
      });
    return doc;
  }
}
